@@ -1,5 +1,7 @@
 #!/usr/bin/perl -w
 
+# construct_dt_and_classify_one_sample.pl
+
 #use lib '../blib/lib', '../blib/arch';
 
 use strict;
@@ -7,39 +9,39 @@ use Algorithm::DecisionTree;
 
 my $training_datafile = "training.dat";
 
-#   COMMENT OUT the debug option below if you do not want to see
-#   each node of the decision being constructed:
+#   COMMENT OUT the two debug options below if you do not
+#   want to see each node of the decision being constructed:
 my $dt = Algorithm::DecisionTree->new( 
                               training_datafile => $training_datafile,
-#                              entropy_threshold => 0.1,
-#                              max_depth_desired => 3,
-#                              debug1 => 1,                          
-    );
+                              entropy_threshold => 0.1,
+                              max_depth_desired => 3,
+#                             debug1            => 1,
+#                             debug2            => 1,
+         );
 
 $dt->get_training_data();
 
 
-#   UNCOMMENT THE NEXT LEFT-JUSTIFIED LINE if you would like to see the
+#   UNCOMMENT THE NEXT STATEMENT if you would like to see the
 #   training data that was read from the disk file:
 #$dt->show_training_data();
 
 
-
 print "\nStarting construction of the decision tree:\n\n";
 my $root_node = $dt->construct_decision_tree_classifier();
 
 
-#   UNCOMMENT THE NEXT LEFT-JUSTIFIED LINE if you would like to see the
+#   UNCOMMENT THE NEXT STATEMENT if you would like to see the
 #   decision tree displayed in your terminal window:
 #$root_node->display_decision_tree("     ");           
 
-
+### The following test_sample is for the training files with names
+### like training.dat training2.dat:
 my @test_sample = qw /exercising=>never 
                       smoking=>heavy 
                       fatIntake=>heavy 
                       videoAddiction=>heavy /;
 
-
 #   The classifiy() in the call below returns a reference to a hash
 #   whose keys are the class labels and the values the associated 
 #   probabilities:
